/**
 * Locale a Module
 * @desczh
 * 对一个模块进行本地化
 * @file
 */
import * as O from 'fp-ts/es6/Option';
import { pipe } from 'fp-ts/es6/pipeable';
import { get, omit } from 'macoolka-object';
import { array } from 'macoolka-collection';
//import { getGlobalI18N } from 'macoolka-env'
const getLocaleDocumentable = (locale) => (a) => {
    return ({
        ...omit(a, ['descriptions']),
        description: locale
            ? pipe(get(a.descriptions, locale), O.fromNullable, O.getOrElse(() => a.description))
            : a.description,
    });
};
function printInterfaceItem(a) {
    const content = '\n  ' + a.signature ? a.signature : '';
    if (a.description.length > 0) {
        let desc = '  /**\n';
        desc += a.description.map(a => `   *${a}`).join('\n');
        desc += '\n   */\n';
        return desc + content;
    }
    return content;
}
function buildInterface(a) {
    let content = `\ninterface ${a.name} ${a.implements.length > 0
        ? `extends ${a.implements.join(',')}`
        : ''}{\n`;
    content += a.fields.map(printInterfaceItem).join('\n');
    content += a.methods.map(printInterfaceItem).join('\n');
    content += '\n}\n';
    return content;
}
export const localeInterface = (locale) => (a) => {
    const _result = {
        ...getLocaleDocumentable(locale)(a),
        fields: pipe(a.fields, array.map(value => getLocaleDocumentable(locale)(value))),
        methods: pipe(a.methods, array.map(value => getLocaleDocumentable(locale)(value))),
    };
    const result = {
        ..._result,
        signature: buildInterface(getLocaleDocumentable(locale)(_result)),
    };
    return result;
};
const localeTypeAlias = (locale) => (a) => {
    const result = {
        ...getLocaleDocumentable(locale)(a),
        fields: pipe(a.fields, array.map(value => getLocaleDocumentable(locale)(value))),
        methods: pipe(a.methods, array.map(value => getLocaleDocumentable(locale)(value))),
    };
    return result;
};
const localeClass = (locale) => (a) => {
    const result = {
        ...getLocaleDocumentable(locale)(a),
        fields: pipe(a.fields, array.map(value => getLocaleDocumentable(locale)(value))),
        methods: pipe(a.methods, array.map(value => getLocaleDocumentable(locale)(value))),
        staticMethods: pipe(a.staticMethods, array.map(value => getLocaleDocumentable(locale)(value))),
    };
    return result;
};
/**
 * Locale a Module.
 * Replace desc'locale' to description
 * @desczh
 * 本地化一个模块
 * 用desc'locale'替换description
 * @since 0.2.0
 */
export const localeModule = (locale) => (a) => {
    const result = {
        ...getLocaleDocumentable(locale)(a),
        constants: pipe(a.constants, array.map(value => getLocaleDocumentable(locale)(value))),
        functions: pipe(a.functions, array.map(value => getLocaleDocumentable(locale)(value))),
        classes: pipe(a.classes, array.map(value => localeClass(locale)(value))),
        interfaces: pipe(a.interfaces, array.map(value => localeInterface(locale)(value))),
        typealiases: pipe(a.typealiases, array.map(value => localeTypeAlias(locale)(value))),
        exports: pipe(a.exports, array.map(value => getLocaleDocumentable(locale)(value))),
    };
    return result;
};
export default localeModule;
//# sourceMappingURL=locale.js.map